library IEEE;
use IEEE.std_logic_unsigned.all;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;
library UNISIM;
use UNISIM.VComponents.all;

entity reset_cont is
port (
    SYS_RST_IN      : in STD_LOGIC;
    SYS_CLK         : in STD_LOGIC;
    SYS_RST         : out STD_LOGIC;

    Reset_req_a     : in STD_LOGIC;
    Reset_req_b     : in STD_LOGIC;
    Reset_req_c     : in STD_LOGIC;
    Reset_req_d     : in STD_LOGIC;
    FPGA_RESET_EN_O : out STD_LOGIC;
    FPGA_RESET_EN_I : in STD_LOGIC;
    FPGA_RESET_EN_T : out STD_LOGIC

);
end reset_cont;

architecture IMP of reset_cont is

    signal counter : STD_LOGIC_VECTOR(0 to 7);
    signal reset_req_lat : STD_LOGIC;
    signal Bus_Reset : STD_LOGIC;
    signal Config_Reset : STD_LOGIC;

begin

SYS_RST <= SYS_RST_IN or Bus_Reset;

FPGA_RESET_EN_O <= Config_Reset;
FPGA_RESET_EN_T <= SYS_RST_IN;

    process(SYS_CLK,SYS_RST_IN)
    begin
        if SYS_RST_IN = '1' then
            counter <= (others => '0');
        elsif SYS_CLK'event and SYS_CLK = '1' then 
            if reset_req_lat = '1' then
                counter <= counter + 1;
            else
                counter <= (others => '0');
            end if;
        end if;
    end process;

    process(SYS_CLK,SYS_RST_IN)
    begin
        if SYS_RST_IN = '1' then
            reset_req_lat <= '0';
            Bus_Reset <= '0';
            Config_Reset <= '0';
        elsif SYS_CLK'event and SYS_CLK = '1' then 
            reset_req_lat <= Reset_req_a or Reset_req_b or Reset_req_c or Reset_req_d or reset_req_lat;
            Bus_Reset <= counter(1) or Bus_Reset;
            Config_Reset <= counter(0) or Config_Reset;
        end if;
    end process;

end IMP;
